/*
*
* Gtk+2.x  Utility routines: File selection widget.
* Copyright (C) 2002-2006 by Tito Ragusa <tito-wolit@tiscali.it>
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License as
* published by the Free Software Foundation; either version 2 of
* the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
* Please read the COPYING and README file!!!
*
* Report bugs to <tito-wolit@tiscali.it>
*
*/

/*#################################### File Selection Widget #####################################################*/

#include <stdio.h>
#include <gtk/gtk.h>

/*
GTK_FILE_CHOOSER_ACTION_OPEN,
GTK_FILE_CHOOSER_ACTION_SAVE,
GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
GTK_FILE_CHOOSER_ACTION_CREATE_FOLDER
*/
char *file_selector(char *title, GtkFileChooserAction fs_action, gboolean local_only, gboolean show_hidden, 
					char *name, char *folder, char *shortcut_folder)
{
	GtkWidget *dialog;
	char *tmp;
	char *filename = NULL;;

	
	tmp = g_locale_to_utf8((title) ? title : "", -1,NULL,NULL,NULL); 	
	dialog = gtk_file_chooser_dialog_new (tmp,
						/*parent_window*/ NULL,
						fs_action,
						GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
						GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
						NULL);
	xfree(&tmp);

	gtk_file_chooser_set_local_only(GTK_FILE_CHOOSER(dialog), local_only);
	gtk_file_chooser_set_show_hidden(GTK_FILE_CHOOSER(dialog), show_hidden);

	/* Returns a GsList *, not a char * */
	/*gtk_file_chooser_set_select_multiple(GTK_FILE_CHOOSER(dialog), select_multiple);*/

	if (name) {
		tmp = g_locale_to_utf8((name) , -1,NULL,NULL,NULL); 	
		gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(dialog), tmp);
		xfree(&tmp);	
	}
	
	if (folder) {
		tmp = g_locale_to_utf8((folder) , -1,NULL,NULL,NULL); 	
		gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(dialog), tmp);
		xfree(&tmp);
	}

	if (shortcut_folder) {
		tmp = g_locale_to_utf8((shortcut_folder) , -1,NULL,NULL,NULL); 	
		gtk_file_chooser_add_shortcut_folder(GTK_FILE_CHOOSER(dialog), tmp, NULL);
		xfree(&tmp);
	}

	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT) {
		filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER (dialog));
	}

	gtk_widget_destroy (dialog);
	return filename;
}

char *default_file_selector(char *title, GtkFileChooserAction fs_action)
{
	return file_selector(title, fs_action, FALSE, TRUE, NULL, NULL, NULL);
}

static void file_sel_ok(GtkWidget *widget, int * filesel_ptr_ok)
{
	DBG_FUNCTION_NAME;
	assert(widget!=NULL);

	SET2(*filesel_ptr_ok, 1);
	gtk_main_quit();
}

char *old_file_selector(char *title, int fileop)
{
	GtkWidget *file_sel_win;
	int ok = 0;
	int * ptr_ok;
	char *selected_path = NULL;

	DBG_FUNCTION_NAME;

	ptr_ok=&ok;

	file_sel_win = gtk_file_selection_new(title);
	/* we only want to open one file selection window at once */
	gtk_window_set_modal(GTK_WINDOW(file_sel_win), TRUE);

	if (fileop)  {/* HIDE_FILE_OP_BUTTONS */
		gtk_file_selection_hide_fileop_buttons(GTK_FILE_SELECTION(file_sel_win));
	}

	/* window placement policy */
	set_position(GTK_WINDOW(file_sel_win), GTK_WIN_POS_CENTER);
	
	/* Set signals */
	signal_connect(GTK_OBJECT(file_sel_win),
				"delete_event",GTK_SIGNAL_FUNC(gtk_main_quit), &file_sel_win);
	signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(file_sel_win)->ok_button),
				"clicked", GTK_SIGNAL_FUNC(file_sel_ok), ptr_ok);
	signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(file_sel_win)->cancel_button),
				"clicked", GTK_SIGNAL_FUNC(gtk_main_quit), &file_sel_win);
	
	gtk_widget_show(file_sel_win);
	gtk_main();

	/* This must be freed later */
	selected_path = g_filename_to_utf8(gtk_file_selection_get_filename(GTK_FILE_SELECTION(file_sel_win)), -1,NULL,NULL,NULL);

	gtk_widget_destroy(file_sel_win);

	/* ok clicked: something was selected */
	if (ok) {
		debug_err_msg("selected path: %s", selected_path);
		return (char *)selected_path;
	}
	/* else nothing was selected  return NULL */
	debug_err_msg("nothing selected");
	
	xfree(&selected_path);
	return NULL;
}

char * default_old_file_selector(char *title)
{
	return old_file_selector(title, SHOW_FILE_OP_BUTTONS);
}
/*############################### End of File Selection Widget #####################################################*/
