/*
*
* Gtk+2.x Utility routines: Permission Window.
* Copyright (C) 2002-2006 by Tito Ragusa <tito-wolit@tiscali.it>
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License as
* published by the Free Software Foundation; either version 2 of
* the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
* Please read the COPYING and README file!!!
*
* Report bugs to <tito-wolit@tiscali.it>
*
*/

#include <gtk/gtk.h>
#include <pwd.h>
#include <grp.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <assert.h>
#include <ctype.h>
#include "mygtk.h"
#include "mylib.h"

struct Perm_data* data;

mode_t get_mode (const char *string)
{
    mode_t mode;

	DBG_FUNCTION_NAME;
	
    if (isdigit (string[0]) )
		return strtoul (string, NULL, 8);
    if (xstrlen (string) != 9) {
		debug_err_msg("bad symbolic mode: %s", string);
		return -1;
    }
    mode = 0;
    if (string[0] == 'r') mode |= S_IRUSR;
    if (string[1] == 'w') mode |= S_IWUSR;
    if (string[2] == 'x') mode |= S_IXUSR;
	if (string[2] == 's') mode |= S_IXUSR|S_ISUID;
	if (string[2] == 'S') mode |= S_ISUID;
	if (string[3] == 'r') mode |= S_IRGRP;
    if (string[4] == 'w') mode |= S_IWGRP;
    if (string[5] == 'x') mode |= S_IXGRP;
	if (string[5] == 's') mode |= S_IXUSR|S_ISGID;
	if (string[5] == 'S') mode |= S_ISGID;
	if (string[6] == 'r') mode |= S_IROTH;
    if (string[7] == 'w') mode |= S_IWOTH;
    if (string[8] == 'x') mode |= S_IXOTH;
	if (string[8] == 't') mode |= S_IXUSR|S_ISVTX;
	if (string[8] == 'T') mode |= S_ISVTX;
	
	debug_err_msg("Mode %s gives %05o", string, mode);
	
	return (mode);
}   /*  End Function get_mode  */

static int (*chown_func)(const char *, uid_t, gid_t) = chown;

/* TODO:
 * -H if a command line argument is a symbolic link to a directory, traverse it
 * -L traverse every symbolic link to a directory encountered
 * -P do not traverse any symbolic links (default)
 */

static int chownAction(const char *fileName, struct stat *statbuf,
		void ATTRIBUTE_UNUSED *junk, int ATTRIBUTE_UNUSED depth)
{
	// TODO: -H/-L/-P
	// if (depth ... && S_ISLNK(statbuf->st_mode)) ....

	if (!chown_func(fileName,
					(data->uid < 0) ? statbuf->st_gid : data->uid,
					(data->gid < 0) ? statbuf->st_gid : data->gid)) {
		debug_err_msg("changed ownership of '%s' to %lu:%lu\n", fileName, 
						(data->uid < 0) ? statbuf->st_gid : data->uid, 
						(data->gid < 0) ? statbuf->st_gid : data->gid);
		return TRUE;
	}
	debug_err_msg("%s: %s", fileName, strerror(errno));	/* A filename can have % in it... */
	return FALSE;
}

static int chmodAction(const char *fileName, struct stat *statbuf, void* junk, int depth)
{
	mode_t newmode;

	/* match coreutils behavior */
	if (depth == 0) {
		/* statbuf holds lstat result, but we need stat (follow link) */
		if (stat(fileName, statbuf))
			goto err;
	} else { /* depth > 0: skip links */
		if (S_ISLNK(statbuf->st_mode))
			return TRUE;
	}
	newmode = statbuf->st_mode;

	if ((newmode = get_mode((char *)junk )) == -1) {
		debug_err_msg("invalid mode: %s", (char *)junk);
		return FALSE;
	}
	if (chmod(fileName, newmode) == 0) {
			debug_err_msg("mode of '%s' changed to %04o (%s)\n", fileName,
							newmode & 07777, bb_mode_string(newmode)+1);
		return TRUE;
	}
 err:
	debug_err_msg("%s: %s", fileName, strerror(errno));
	return FALSE;
}

void set_r_mode_callback(GtkToggleButton *widget, char *position)
{
	assert(widget != NULL);
	
	DBG_FUNCTION_NAME;
	
	if (gtk_toggle_button_get_active (widget)==TRUE)
		position[0] = 'r';
	else
		position[0] = '-';
}

void set_w_mode_callback(GtkToggleButton *widget, char *position)
{
	assert(widget != NULL);

	DBG_FUNCTION_NAME;
	
	if (gtk_toggle_button_get_active (widget)==TRUE)
		position[0] = 'w';
	else
		position[0] = '-';
}

void set_x_mode_callback(GtkToggleButton *widget, char *position)
{
	assert(widget != NULL);
	
	DBG_FUNCTION_NAME;
	
	if (gtk_toggle_button_get_active (widget) == TRUE) {
		switch(position[0]) {
			case '-':
				position[0] = 'x';
				break;
			case 'S':
				position[0] = 's';
				break;
			case 'T':
				position[0] = 't';
				break;
		}
	} else {
		switch(position[0]) {
			case 's':
				position[0] = 'S';
				break;
			case 't':
				position[0] = 'T';
				break;
			case 'x':
				position[0] = '-';
				break;
		}
	}
}

void set_s_mode_callback(GtkToggleButton *widget, char *position)
{
	assert(widget != NULL);

	DBG_FUNCTION_NAME;
	
	if (gtk_toggle_button_get_active (widget) == TRUE) {
		switch(position[0]) {
			case '-':
				position[0] = 'S';
				break;
			case 'x':
				position[0] = 's';
				break;
		}
	} else {
		switch(position[0]) {
			case 's':
				position[0] = 'x';
				break;
			case 'S':
				position[0] = '-';
				break;
		}
	}
}

void set_t_mode_callback(GtkToggleButton *widget, char *position)
{
	assert(widget != NULL);
	
	DBG_FUNCTION_NAME;
	
	if (gtk_toggle_button_get_active (widget) == TRUE) {
		switch(position[0]) {
			case '-':
				position[0] = 'T';
				break;
			case 'x':
				position[0] = 't';
				break;
		}
	} else {
		switch(position[0]) {
			case 't':
				position[0] = 'x';
				break;
			case 'T':
				position[0] = '-';
				break;
		}
	}
}

void recursive_callback(GtkToggleButton *widget, int *ptr_recursive)
{
	assert(widget != NULL);
	DBG_FUNCTION_NAME;
	
	if (gtk_toggle_button_get_active (widget) == TRUE)
		SET2(*ptr_recursive,1);
	else
		SET2(*ptr_recursive,0);
}

void perm_quit_callback(GtkWidget *widget,  Perm_data* d)
{
	DBG_FUNCTION_NAME;

	assert(widget != NULL);

	SET2(*d->ptr_dialog_retval, 1);
	d->user = xstrdup(gtk_entry_get_text (GTK_ENTRY (d->user_entry)));
	d->group = xstrdup(gtk_entry_get_text (GTK_ENTRY (d->group_entry)));
	gtk_main_quit();
	gtk_widget_destroy(d->window);
}

int PermissionWindow(char *filename)
{
	GtkWidget *table;
	GtkWidget *permissions_table;
	GtkWidget *ownership_table;
	GtkWidget *button;
	GtkWidget *bbox;
	GtkWidget *frame;
	GtkWidget *label;
	GtkWidget *checkbox;
	char mode[11];
	char mode_saved[11];
	char *endptr;
	int  recursive = 0;
	int *ptr_recursive;
	int  execute = 0;
	int status = 0;
	int errno_save;
	struct stat fileinfo;
	struct group *gps;
	struct passwd *pws;

	DBG_FUNCTION_NAME;

	if (file_exists(filename, &fileinfo) < 0) {
		dialog_window(STR_ERROR, ONE_BUTTON, NULL, 1, STR_UNABLE_TO_STAT_S_S, filename, strerror(errno));
		return 1;
	}
	ptr_recursive = &recursive;
	data = (Perm_data*)xmalloc(sizeof(Perm_data));
	data->ptr_dialog_retval = &execute;
	data->window = window_create( 380, 400, 8, filename, GTK_SIGNAL_FUNC(gtk_main_quit), NULL, MODAL);
	data->user = NULL;
	data->group = NULL;
	table = table_create(8, 12, data->window , 5,NOT_HOMOGENEUS);
	
	xsnprintf(mode, 11, "%s", bb_mode_string(fileinfo.st_mode));
	debug_err_msg("%s() mode: %s", __FUNCTION__, mode);
	
	if (mode == NULL || xstrlen(mode) != 10) {
		dialog_window(STR_ERROR, ONE_BUTTON, NULL, 1, STR_S_BAD_MODE_STRING_S, filename,
														(mode == NULL) ? "(NULL)" : mode);
		return 1;
	}

	xsnprintf(mode_saved, 11, "%s", mode);
	debug_err_msg("%s() mode_saved: %s", __FUNCTION__, mode_saved);	
	
	frame = frame_create_on_table(STR_ACCESS_PERMISSIONS, GTK_WIDGET(table), 0, 8, 0, 6);
	permissions_table = table_create(12, 8, frame , 5, NOT_HOMOGENEUS);
	if (S_ISLNK(fileinfo.st_mode)) {
		/* if is a link skip access permissions */
		label = default_label_create_on_table(STR_FILE_IS_A_LINK,	GTK_WIDGET(permissions_table), 0, 12, 0, 1);
	} else {
		label = default_label_create_on_table(STR_CLASS,	GTK_WIDGET(permissions_table), 0, 2, 0, 1);
		if ( S_ISDIR(fileinfo.st_mode)) {
			/* if is a dir change access permissions text */
			label = default_label_create_on_table(STR_SHOW_ENTRIES,	GTK_WIDGET(permissions_table), 2, 4, 0, 2);
			label = default_label_create_on_table(STR_WRITE_ENTRIES,	GTK_WIDGET(permissions_table), 4, 6, 0, 2);
			label = default_label_create_on_table(STR_ENTER,			GTK_WIDGET(permissions_table), 6, 8, 0, 1);
		} else {
			label = default_label_create_on_table(STR_READ,	GTK_WIDGET(permissions_table), 2, 3, 0, 1);
			label = default_label_create_on_table(STR_WRITE,	GTK_WIDGET(permissions_table), 4, 5, 0, 1);
			label = default_label_create_on_table(STR_EXEC,	GTK_WIDGET(permissions_table), 6, 7, 0, 1);
		}
		label = default_label_create_on_table(STR_SPECIAL,	GTK_WIDGET(permissions_table),8, 10, 0, 1);
		label = default_label_create_on_table(STR_USER,		GTK_WIDGET(permissions_table),0,  2, 2, 3);
		checkbox = check_create_on_table(NULL,GTK_SIGNAL_FUNC(set_r_mode_callback),&mode[1],
										(mode[1] == 'r') ? ACTIVE : INACTIVE,
										GTK_WIDGET(permissions_table), 2, 3, 2, 3 );
		checkbox = check_create_on_table(NULL, GTK_SIGNAL_FUNC(set_w_mode_callback), &mode[2],
										(mode[2] == 'w') ? ACTIVE : INACTIVE,
										GTK_WIDGET(permissions_table), 4, 5, 2, 3 );
		checkbox = check_create_on_table(NULL, GTK_SIGNAL_FUNC(set_x_mode_callback), &mode[3],
										((mode[3] == 'x') || (mode[3] == 's')) ? ACTIVE : INACTIVE,
										GTK_WIDGET(permissions_table), 6,  7, 2, 3 );
		checkbox = check_create_on_table(NULL, GTK_SIGNAL_FUNC(set_s_mode_callback), &mode[3],
										((mode[3] == 'S') || (mode[3] == 's')) ? ACTIVE : INACTIVE,
										GTK_WIDGET(permissions_table), 9, 10, 2, 3 );
		label = default_label_create_on_table(STR_SET_UID, GTK_WIDGET(permissions_table), 10, 12, 2 ,3);
		label = default_label_create_on_table(STR_GROUP, GTK_WIDGET(permissions_table),  0,  2, 4, 5);
		checkbox = check_create_on_table(NULL, GTK_SIGNAL_FUNC(set_r_mode_callback), &mode[4],
										(mode[4] == 'r') ? ACTIVE : INACTIVE,
										GTK_WIDGET(permissions_table), 2,  3, 4, 5 );
		checkbox = check_create_on_table(NULL, GTK_SIGNAL_FUNC(set_w_mode_callback), &mode[5],
										(mode[5] == 'w') ? ACTIVE : INACTIVE,
										GTK_WIDGET(permissions_table), 4,  5, 4, 5 );
		checkbox = check_create_on_table(NULL, GTK_SIGNAL_FUNC(set_x_mode_callback), &mode[6],
									((mode[6] == 'x') || (mode[6] == 's')) ? ACTIVE : INACTIVE,
									GTK_WIDGET(permissions_table), 6,  7, 4, 5 );
		checkbox = check_create_on_table(NULL, GTK_SIGNAL_FUNC(set_s_mode_callback), &mode[6],
									((mode[6] == 'S') || (mode[6] == 's')) ? ACTIVE : INACTIVE,
									GTK_WIDGET(permissions_table), 9, 10, 4, 5 );
		label = default_label_create_on_table(STR_SET_GID, GTK_WIDGET(permissions_table), 10, 12, 4 ,5);
		label = default_label_create_on_table(STR_OTHERS,  GTK_WIDGET(permissions_table),  0, 2, 6, 7);
		checkbox = check_create_on_table(NULL, GTK_SIGNAL_FUNC(set_r_mode_callback), &mode[7],
										(mode[7] == 'r') ? ACTIVE : INACTIVE,
										GTK_WIDGET(permissions_table), 2,  3, 6, 7 );
		checkbox = check_create_on_table(NULL, GTK_SIGNAL_FUNC(set_w_mode_callback), &mode[8],
										(mode[8] == 'w') ? ACTIVE : INACTIVE,
										GTK_WIDGET(permissions_table), 4,  5, 6, 7 );
		checkbox = check_create_on_table(NULL, GTK_SIGNAL_FUNC(set_x_mode_callback), &mode[9],
									((mode[9] == 'x') || (mode[9] == 't')) ? ACTIVE : INACTIVE,
									GTK_WIDGET(permissions_table), 6,  7, 6, 7 );
		checkbox = check_create_on_table(NULL, GTK_SIGNAL_FUNC(set_t_mode_callback), &mode[9],
									((mode[9] == 't') || (mode[9] == 'T')) ? ACTIVE : INACTIVE,
									GTK_WIDGET(permissions_table), 9, 10, 6, 7 );
		label = default_label_create_on_table(STR_STICKY,	GTK_WIDGET(permissions_table),10, 12, 6 ,7);
	}
	frame = frame_create_on_table(STR_OWNERSHIP, GTK_WIDGET(table),0,8, 6, 9);
	ownership_table = table_create(5, 2, frame , 5,NOT_HOMOGENEUS);
	label = default_label_create_on_table(STR_USER":", GTK_WIDGET(ownership_table),0, 1, 0 ,1);
	data->user_entry = default_text_box_create_on_table( get_uid_or_gid(UID, fileinfo.st_uid), EDITABLE,
												GTK_WIDGET(ownership_table), 1, 5, 0, 1);
	label = default_label_create_on_table(STR_GROUP":",	GTK_WIDGET(ownership_table),0, 1, 1 ,2);
	data->group_entry = default_text_box_create_on_table(get_uid_or_gid(GID, fileinfo.st_gid), EDITABLE,
												GTK_WIDGET(ownership_table), 1, 5, 1, 2);

	if (S_ISDIR(fileinfo.st_mode)) {
		/* if is a dir show checkbox for recursive action */
		checkbox = check_create_on_table(STR_APPLY_TO_SUBDIRS, GTK_SIGNAL_FUNC(recursive_callback),
										ptr_recursive , INACTIVE,
										GTK_WIDGET(table), 0, 8, 9, 10 );
	}

	bbox = default_button_box_create_on_table(HORIZONTAL, table, 0, 8, 11, 12);
	button = button_create_in_container(STR_OK, GTK_SIGNAL_FUNC(perm_quit_callback), data, bbox);
	button = button_create_in_container(STR_CANCEL, GTK_SIGNAL_FUNC(window_close), data->window, bbox);

	main_loop(&data->window);

	if (execute == 0) {
		debug_err_msg("%s(): execute %d",__FUNCTION__, execute);
		xfree(&data);
		return 0;
	}

	if (strcmp(mode_saved, mode) != 0) {
		/* if mode changed apply */
		debug_err_msg("%s(): mode changed", __FUNCTION__);
		/* We need to skip the first char of mode ( - or d ) here so we pass a pointer to mode[1] as arg */
		if (recursive_action (filename, recursive, FALSE, FALSE, chmodAction ,chmodAction , &mode[1], 0) == FALSE) {
			dialog_window(STR_ERROR, ONE_BUTTON, NULL, 1, STR_ERROR_CHANGING_MODE_FOR_S_S, filename, strerror(errno));
			debug_err_msg("%s(): error changing  mode for: %s",__FUNCTION__, filename);
			goto END;
		}
		status = 1;
	}

	/* if ownership changed apply */
	if (strcmp(data->user,  get_uid_or_gid(UID, fileinfo.st_uid)) !=0
		||strcmp(data->group, get_uid_or_gid(GID, fileinfo.st_gid)) !=0 ) {
		debug_err_msg("%s(): user or group changed", __FUNCTION__);

		if ((gps = getgrnam(data->group)) != NULL) {
			data->gid = gps->gr_gid;
		} else {
			errno_save = errno;
			errno = 0;
			data->gid = strtol(data->group, &endptr, 10);
			if (errno != 0 || *endptr != '\0' || endptr == data->group 
			|| data->gid < LONG_MIN || data->gid > LONG_MAX) {
				dialog_window(STR_ERROR, ONE_BUTTON, NULL, 1, STR_BAD_GROUPNAME_S, data->group);
				goto END;
			}
			errno = errno_save;
		}
		
		if ((pws = getpwnam(data->user)) !=NULL ) {
			data->uid = pws->pw_uid;
		} else {
			errno_save = errno;
			errno = 0;
			data->uid = strtol(data->user, &endptr, 10);
			if (errno != 0 || *endptr!='\0' || endptr == data->user || data->uid < LONG_MIN || data->uid > LONG_MAX) {
				dialog_window(STR_ERROR, ONE_BUTTON, NULL, 1, STR_BAD_USERNAME_S, data->user);
				goto END;
			}
			errno = errno_save;
		}
		
		debug_err_msg("%s(): old gid %d new gid %ld old uid %d new uid %ld", __FUNCTION__,
								fileinfo.st_gid, data->gid, fileinfo.st_gid, data->uid);

		if (recursive_action (filename, recursive , FALSE, FALSE, chownAction, chownAction, NULL, 0) == FALSE) {
			dialog_window(STR_ERROR, ONE_BUTTON, NULL, 1, STR_ERROR_CHANGING_OWNERSHIP_FOR_S_S, filename, strerror(errno));
			debug_err_msg("%s(): error changing user or group for: %s",__FUNCTION__, filename);
		}
		status = 1;
	} 
END:	
	/* Don't free them!!! */
	/*xfree(&data->user);*/
	/*xfree(&data->group);*/
	/*data->uid = 0;*/
	/*data->gid = 0;*/
	xfree(&data);
	return status;
}
 
