#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* get_line_from_file() - This function reads an entire line from a text file,
 * up to a newline. It returns a malloc'ed char * which must be stored and
 * free'ed  by the caller.  If 'c' is nonzero, the trailing '\n' (if any)
 * is removed.  In event of a read error or EOF, NULL is returned. */

static char *private_get_line_from_file(FILE *file, int c)
{
#define GROWBY (80)		/* how large we will grow strings by */

	int ch;
	int idx = 0;
	char *linebuf = NULL;
	int linebufsz = 0;

	while ((ch = getc(file)) != EOF) {
		/* grow the line buffer as necessary */
		if (idx > linebufsz - 2) {
			linebuf = realloc(linebuf, linebufsz += GROWBY);
		}
		linebuf[idx++] = (char)ch;
		if (ch == '\n' || ch == '\0') {
			if (c) {
				--idx;
			}
			break;
		}
	}
	if (linebuf) {
		if (ferror(file)) {
			free(linebuf);
			return NULL;
		}
		linebuf[idx] = 0;
	}
	return linebuf;
}

extern char *bb_get_line_from_file(FILE *file)
{
	return private_get_line_from_file(file, 0);
}

extern char *bb_get_chomped_line_from_file(FILE *file)
{
	return private_get_line_from_file(file, 1);
}


int main(int argc, char **argv)
{
	FILE *file;
	FILE *output;
	char *lineptr=NULL;
	
	if(argc<2){
		printf("missing arg\n");
		exit(1);
	}
	
	if((file=fopen(argv[1], "r"))==NULL) {
		printf("cannot open file: %s: %m\n",argv[1]);
		exit(1);
	}
	
	while((lineptr=bb_get_line_from_file(file))!=NULL)
	{
		if(lineptr[0]=='/' || lineptr[0]=='*' || lineptr[0]=='\n'  || lineptr[0]==' ')
		{
			/* VOID */;
		}
		else if(lineptr[0]=='#' || lineptr[0]=='\t')
		{
			while (lineptr[0]!='"')
				lineptr++;
			printf("%s",lineptr);
			/* remove Newline */
			lineptr[strlen(lineptr)-1]='\0';
			if((output=fopen("check.c", "w+"))==NULL)
			{
				printf("cannot open file: %s: %m\n",argv[1]);
				exit(1);
			}
			fprintf(output, "%s\n","#include <stdio.h>");
			fprintf(output, "%s\n","#include <stdlib.h>");
			fprintf(output, "%s\n","int main(int argc, char **argv)");
			fprintf(output, "%s\n","{");
			fprintf(output, "%s\n","gtk_init (&argc, &argv);");
			fprintf(output,"%s%s%s\n", "dialog_window(NULL, ONE_BUTTON, NULL, 1,", lineptr,",\"x\",\"x\",\"x\",\"x\");");
			fprintf(output,"return(0);\n%s\n\n","}");
			fclose(output);
			system("make");
			system("./gtk_check");
	
		}
		else
		{
			printf("unknown error\n");
		}
	}
	exit(0);
} 
